@@ -1,7 +1,8 @@
 from pydbops.pydbops import pydbops
 from pydbops.tables import Table
-from typing import overload
 from pydbops.UserDefinedExceptions import NoSuchTableError
+import sqlite3
+from typing import overload
 
 
 class Database(pydbops):
@@ -19,18 +20,23 @@ class Database(pydbops):
         - databaseVersion() - Returns sqlite3 version.
         - dropTable() - Function for deleting table.
         - fetchInOrder() - Function for fetching database entries in given order.
+        - getData() - Function for getting all data.
         - getFieldNames() - Function for getting field names.
+        - getTable() - Creates Table instance.
+        - intersection() - Performs intersection and returns all distinct rows selected by query.
         - length() - Returns length of table. Returns 0 if table is empty.
+        - minus() - Fetches rows which are present in first query but absent in second.
         - removeEntry() - Function for removing records from table.
         - searchEntry() - Function for searching in table.
         - tableNames() - Accesses tables in a database.
+        - union() - Performs union and returns all distinct rows selected by query.
         - updateEntry() - Function for updating values in table.
-        - values() - Accesses records in a table.
     """
     def __init__(self, filepath: str) -> None:
         super().__init__(filepath)
         self.__filepath = filepath
-        self.tables = self.tableNames(count=False, list=True, dictionary=False)
+        self.tables: list[str] = Database.tableNames(self, count=False, list=True, dictionary=False)
+        self.data:dict[str, dict[str, list[tuple]]] = Database.getData(self)
 
     def __str__(self) -> str:
         string = ""
@@ -53,8 +59,39 @@ class Database(pydbops):
         Returns:
             id of the entry inserted.
         """
+        self.data: dict[str, dict[str, list[tuple]]] = Database.getData(self)
         return super().addEntry(table, values)
 
+    @overload
+    def createIndex(self, indexName: str, tableName: str, columnName: str, unique: bool = False) -> bool: ...
+
+    @overload
+    def createIndex(self, indexName: str, tableName: str, columnName: list[str], unique: bool = False) -> bool: ...
+
+    def createIndex(self, indexName: str, tableName: str, columnName: str | list[str], unique: bool = False) -> bool:
+        """
+        Creates index of given column.
+
+        Args:
+            - indexName (str) : Name of index.
+            - tableName (str) : Name of table.
+            - columnName (str) : Name of column in table.
+            - unique (bool) : If true, unique index will be created.
+
+        Returns:
+            - True if executed.
+        """
+        conn = sqlite3.connect(self.__filepath)
+        c = conn.cursor()
+        if unique:
+            c.execute(f"""CREATE UNIQUE INDEX {indexName}\nON {tableName}({columnName})""")
+        else:
+            c.execute(f"""CREATE INDEX {indexName}\nON {tableName}({columnName})""")
+        conn.commit()
+        conn.close()
+        self.data: dict[str, dict[str, list[tuple]]] = Database.getData(self)
+        return True
+
     def createTable(self, tableName: str, fields: dict[str, str]) -> bool:
         """
         Creates table of given name.
@@ -65,9 +102,16 @@ class Database(pydbops):
         Returns:
             - True if executed.
         """
-        ret = super().createTable(tableName, fields)
-        self.tables = self.tableNames(count=False, list=True, dictionary=False)
-        return ret
+        columns = "(" + ", \n".join(["{} {}".format(k, v) for k, v in fields.items()]) + ")"
+        conn = sqlite3.connect(self.__filepath)
+        c = conn.cursor()
+        c.execute(f"""CREATE TABLE IF NOT EXISTS {tableName} \n {columns}""")
+        conn.commit()
+        conn.close()
+        self._table = tableName
+        self.tables.append(tableName)
+        self.data: dict[str, dict[str, list[tuple]]] = Database.getData(self)
+        return True
 
     def databaseVersion(self) -> str:
         """
@@ -85,6 +129,7 @@ class Database(pydbops):
 
         Returns: list of records.
         """
+        self.data: dict[str, dict[str, list[tuple]]] = Database.getData(self)
         ret = super().dropTable(table=table, getData=getData)
         self.tables = self.tableNames(count=False, list=True, dictionary=False)
         return ret
@@ -114,6 +159,26 @@ class Database(pydbops):
         """
         return super().fetchInOrder(table=table, field=field)
 
+    def getData(self) -> dict[str, dict[str, list[tuple]]]:
+        """
+        Function for getting all data.
+        """
+        conn = sqlite3.connect(self.__filepath)
+        c = conn.cursor()
+        mydata: dict[str, dict[str, list[tuple]]] = {}
+        for table in self.tables:
+            mydata[table] = {}
+            column_names = Database.getFieldNames(self, table=table, returnType="list")
+            for column in column_names:
+                c.execute(f"SELECT {column} from {table}")
+                records: list[tuple] = c.fetchall()
+                rec_list: list[str] = []
+                for record in records:
+                    rec_list.append(record[0])
+                mydata[table][column] = rec_list
+        c.close()
+        return mydata
+
     @overload
     def getFieldNames(self, table: str, returnType: str = "list") -> list[str]: ...
 
@@ -149,6 +214,28 @@ class Database(pydbops):
         else:
             raise(NoSuchTableError(table=table))
 
+    def intersection(self, tableName1: str, tableName2: str, column_name: str) -> list[tuple[str | int]]:
+        """
+        Performs intersection and returns all distinct rows selected by query.
+
+        Args:
+            - tableName1 (str) : Name of table 1
+            - tableName2 (str) : Name of table 2
+            - column_name (str) : Name of column on which intersection is to be performed.
+
+        Returns:
+            - list of records.
+        """
+        conn = sqlite3.connect(self.__filepath)
+        c = conn.cursor()
+        c.execute(f"SELECT {column_name} from {tableName1} INTERSECT SELECT {column_name} from {tableName2}")
+        records = c.fetchall()
+        c.close()
+        records_list = []
+        for record in records:
+            records_list.append(record[0])
+        return records_list
+
     def length(self) -> int:
         """
         Checks if database is empty.
@@ -157,6 +244,28 @@ class Database(pydbops):
         """
         return super().length()
 
+    def minus(self, tableName1: str, tableName2: str, column_name: str) -> list[tuple[str | int]]:
+        """
+        Fetches rows which are present in first query but absent in second. ("except" operation)
+
+        Args:
+            - tableName1 (str) : Name of table 1
+            - tableName2 (str) : Name of table 2
+            - column_name (str) : Name of column on which operation is to be performed.
+
+        Returns:
+            - list of records.
+        """
+        conn = sqlite3.connect(self.__filepath)
+        c = conn.cursor()
+        c.execute(f"SELECT {column_name} from {tableName1} EXCEPT SELECT {column_name} from {tableName2}")
+        records = c.fetchall()
+        c.close()
+        records_list = []
+        for record in records:
+            records_list.append(record[0])
+        return records_list
+
     def removeEntry(self, table: str, id: int = -1, keyword: str = "", deleteAllOccurences: bool = False, deleteAll: bool = False) -> bool:
         """
         Function for removing records from database.
@@ -171,7 +280,9 @@ class Database(pydbops):
         Returns:
             True if deleted a record. False if record not found.
         """
-        return super().removeEntry(table, id, keyword, deleteAllOccurences, deleteAll)
+        ret = super().removeEntry(table, id, keyword, deleteAllOccurences, deleteAll)
+        self.data: dict[str, dict[str, list[tuple]]] = Database.getData(self)
+        return ret
 
     @overload
     def searchEntry(self, table: str, id: int = -1, keyword: str = "", returnType: str = "ids", findAllOccurence: bool = False) -> int: ...
@@ -216,6 +327,28 @@ class Database(pydbops):
         """
         return super().tableNames(count, list, dictionary)
 
+    def union(self, tableName1: str, tableName2: str, column_name: str) -> list[tuple[str | int]]:
+        """
+        Performs union and returns all distinct rows selected by query.
+
+        Args:
+            - tableName1 (str) : Name of table 1
+            - tableName2 (str) : Name of table 2
+            - column_name (str) : Name of column on which union is to be performed.
+
+        Returns:
+            - list of records.
+        """
+        conn = sqlite3.connect(self.__filepath)
+        c = conn.cursor()
+        c.execute(f"SELECT {column_name} from {tableName1} UNION SELECT {column_name} from {tableName2}")
+        records = c.fetchall()
+        c.close()
+        records_list = []
+        for record in records:
+            records_list.append(record[0])
+        return records_list
+
     @overload
     def updateEntry(self, table: str, values: dict[str, str | int], whereField: str, Is: int) -> bool: ...
 
@@ -231,15 +364,23 @@ class Database(pydbops):
             values (dict): key is field name and value is value to be updated.
             field (str) : field name to be checked for entry to be updated.
             whereFieldIs (str | int) : field value to be checked.
+
         Returns:
             id of the entry inserted.
         """
-        return super().updateEntry(table, values, whereField, Is)
+        ret = super().updateEntry(table, values, whereField, Is)
+        self.data: dict[str, dict[str, list[tuple]]] = Database.getData(self)
+        return ret
 
 
 def openDatabase(filename: str) -> Database:
     """
     Creates a database and returns a Database object.
     """
-    d = Database(filepath=filename)
-    return d
+    if filename[-3:] != ".db":
+        raise(FileNotFoundError)
+    try:
+        d = Database(filepath=filename)
+        return d
+    except FileNotFoundError:
+        raise(FileNotFoundError(filename))
